
// does nothing only for initialization before content is set
// copyright 2024 mirko boeer

tinymce.PluginManager.add('initialization', (editor, url) => {

  editor.on('BeforeSetContent',
    function(contents){
      // fullpage plugin also calls it
      AddOwnHTMLTagsAndAttribs( contents.target );
    }
  );

  return {
    name: 'InitializationPlugin',
    url: 'https://www.superwebmailer.de/'
  };

});

// fullpage plugin calls it before page is loaded in editor!!
// add attributs for target groups, tinymce removes unknown attributs
// add IPE tags
function AddOwnHTMLTagsAndAttribs(editor){

    // https://www.tiny.cloud/docs/tinymce/6/apis/tinymce.html.schema/

    if(editor && editor.target)
      editor = editor.target;

    if(!editor || !editor.schema) return;

    var Rule = editor.schema.getElementRule(supported_targetgroups_Elements[0].toLowerCase());

    if( !Rule.attributes.target_groups ){
      Rule.attributes["target_groups"] =  {};
      Rule.attributesOrder.push("target_groups");

      for(var i=1; i<supported_targetgroups_Elements.length; i++){
        Rule = editor.schema.getElementRule(supported_targetgroups_Elements[i].toLowerCase());
        if(Rule && !Rule.attributes.target_groups){
          Rule.attributes["target_groups"] =  {};
          Rule.attributesOrder.push("target_groups");
        }
      }
    }

    // IPE elements Templates edit, other editors will remove IPE tags
    if(document.getElementById("TemplatesForm") || document.getElementsByName("TemplateId").length > 0){
      const ipe_Tags = ["singleline", "multiline", "tableofcontents", "tableofcontentstitle", "repeater"];

      const e = editor;
      var Rule = e.schema.getElementRule(ipe_Tags[0]);
      if(!Rule){
        for(var i=0; i<ipe_Tags.length; i++){
          e.schema.addCustomElements(ipe_Tags[i]);
          Rule = e.schema.getElementRule(ipe_Tags[i]);
          if(Rule){
             Rule.attributes["target_groups"] =  {};
             Rule.attributesOrder.push("target_groups");
             Rule.attributes["label"] = {};
             Rule.attributesOrder.push("label");
          }
        }

        Rule = e.schema.getElementRule("singleline");
        Rule.attributes["tableofcontentstitle"] = {};
        Rule.attributesOrder.push("tableofcontentstitle");

        Rule = e.schema.getElementRule("repeater");
        Rule.attributes["editable"] = {};
        Rule.attributesOrder.push("editable");
        Rule.attributes["disableOtherElements"] = {};
        Rule.attributesOrder.push("disableOtherElements");

        let realyaddValidChildren = function (tagName, Or_childtagNames){ // sample tagName: A, Or_childtagNames: singleline|multiline|repeater
           var childs = e.schema.children[tagName];
           var s = Or_childtagNames;
           if(childs)
             for (const [tag, value] of Object.entries(childs)) { //enum other tags
               s += "|" + tag;
             };
           e.schema.addValidChildren(tagName + "[" + s + "]"); //TinyMCE removes all and adds this
        }
        realyaddValidChildren("ul", "tableofcontents");
        realyaddValidChildren("ol", "tableofcontents");
        realyaddValidChildren("tableofcontents", "li");
        realyaddValidChildren("li", "tableofcontentstitle");

        var elements = e.schema.getTextBlockElements()
        for (const [tag, value] of Object.entries(elements)) {
          if(tag.toUpperCase() == tag) continue;
          realyaddValidChildren(tag, "singleline|multiline|repeater");
        }

        var elements = e.schema.getTextInlineElements()
        for (const [tag, value] of Object.entries(elements)) {
          if(tag.toUpperCase() == tag) continue;
          realyaddValidChildren(tag, "singleline|multiline|repeater");
        }

        const other_ipeTags = ['img', 'hr', 'a', 'ol', 'ul'];

        for(var i=0; i<other_ipeTags.length; i++){
          Rule = e.schema.getElementRule(other_ipeTags[i]);
          Rule.attributes["editable"] = {};
          Rule.attributesOrder.push("editable");
          Rule.attributes["label"] = {};
          Rule.attributesOrder.push("label");
        }

      }
    }

}

