// qrcode for SuperWebMailer/SuperMailingList
// copyright 2024 mirko boeer


var defaultQRCodeImage = "images/qrcode.png";

tinymce.PluginManager.add('qrcode', (editor, url) => {

  editor.ui.registry.addIcon('qrcode', DownloadFile(tinymce.defaulteditorConfig.baseMyPluginsUrl + 'qrcode/qrcode.svg'));

  let language = editor.options.get("language");
  var qrcode_notAllowedTags = ["IMG", "TABLE", "TR", "HR", "OBJECT", "EMBED"];


  var QRCodetooltip = language == "de" ? HTMLEntityDecode('QR-Code einf&uuml;gen/bearbeiten') : 'Insert/edit QR code';
  var QRCodeTitle = language == "de" ? HTMLEntityDecode('QR-Code einf&uuml;gen/bearbeiten...') : 'Insert/edit QR code...';

  var qrcodeAction = function(){

         function _encodeURI(s){
           s = encodeURI(s);
           s = s.replace(/\&amp\;/gi, '%26').replace(/\&/gi, '%26');
           return s;
         }

         let PlaceholderItems = GetAllDefinedSWMSMLPlaceholders(editor, []);
         let p = Array();
         let defaultPlaceholderText = editor.getParam("language", "en") == "de" ? "Platzhalter..." : "Placeholders...";
         p = p.concat( { value: "", text: defaultPlaceholderText } );

         for(var i=0; i<PlaceholderItems.length; i++){
           if( PlaceholderItems[i][0].indexOf("Link") > 0 ) continue;
           p = p.concat( { value: PlaceholderItems[i][0], text: pluginHTMLEntityDecode( PlaceholderItems[i][1] ) } );
         }

         var linkParams = {};
         element = editor.selection.getStart();
         if(element && element.tagName == "IMG" && element.src.indexOf(defaultQRCodeImage + '?') >= 0){
           const searchParams = new URLSearchParams( element.src.substr( element.src.indexOf(defaultQRCodeImage + '?') + defaultQRCodeImage.length + 1 ) );
           for (const [key, value] of searchParams) {
             linkParams[key] = value;
           }
         }

         var params = {};
         params.dimension = linkParams.dimension ? linkParams.dimension : "100";
         params.content = linkParams.content ? linkParams.content : 'Text';
         params.eccLevel = linkParams.eccLevel ? linkParams.eccLevel : "Q";
         params.outputType = linkParams.outputType ? linkParams.outputType : "png";
         params.asDataURI = linkParams.asDataURI ? linkParams.asDataURI === "true" : false;
         params.outputBase64 = linkParams.outputBase64 ? linkParams.outputBase64 === "true" : false;
         params.quietzoneSize = linkParams.quietzoneSize ? linkParams.quietzoneSize : "1";

         // https://www.tiny.cloud/docs/tinymce/6/dialog/
         editor.windowManager.open({
                title: QRCodeTitle,
                size: "normal",
                body: {
                    type: "panel",
                    items: [
                      {
                         name: "dimension",
                         type: "input",
                         inputMode: 'numeric',
                         label: pluginHTMLEntityDecode( editor.options.get("language") == "de" ? "Gr&ouml;&szlig;e in Pixel" : "Size in pixels" ),
                         placeholder: "Pixels"
                      },
                      {
                         name: "eccLevel",
                         type: "selectbox",
                         label: pluginHTMLEntityDecode( editor.options.get("language") == "de" ? "Fehlerkorrektur" : "Error correction" ),
                         enabled: true,
                         size: 1,
                         items: [
                           { value: 'L', text: 'L' },
                           { value: 'M', text: 'M' },
                           { value: 'Q', text: 'Q' },
                           { value: 'H', text: 'H' }
                         ]
                      },
                      {
                         name: "outputType",
                         type: "selectbox",
                         label: pluginHTMLEntityDecode( editor.options.get("language") == "de" ? "Dateiformat" : "File format" ),
                         enabled: true,
                         size: 1,
                         items: [
                           { value: 'png', text: 'PNG' },
                           { value: 'jpg', text: 'JPEG' },
                           { value: 'gif', text: 'GIF' }
                         ]
                      },
                      {
                        type: 'checkbox',
                        name: 'asDataURI',
                        label: pluginHTMLEntityDecode( editor.options.get("language") == "de" ? 'Als DATA-URI base64-codiert einf&uuml;gen' : "Insert as DATA URI base64 encoded" ),
                        enabled: true
                      },

                      {
                         name: "quietzoneSize",
                         type: "input",
                         inputMode: 'numeric',
                         label: pluginHTMLEntityDecode( editor.options.get("language") == "de" ? "Gr&ouml;&szlig;e Rahmen Pixel" : "Size of border in pixels" ),
                         placeholder: "Pixels"
                      },

                      {
                        type: 'selectbox',
                        name: 'placeholders',
                        label: pluginHTMLEntityDecode( editor.options.get("language") == "de" ? "Platzhalter in Text f&uuml;r QR-Code einf&uuml;gen" : "Insert placeholder in text for QR code" ),
                        enabled: true,
                        size: 1,
                        items: p
                      },

                      {
                         name: "content",
                         type: "textarea",
                         label: pluginHTMLEntityDecode( editor.options.get("language") == "de" ? "Text f&uuml;r QR-Code, sf_url_encode() f&uuml;r URL-Parameter verwenden" : "Pain text for QR code, use sf_url_encode() for URL parameters" ),
                         maximized: true
                      },
                      {
                        type: 'label',
                        label: 'Preview',
                        items: [

                                           {
                                              type: 'iframe',
                                              name: 'qrcodepreview',
                                              height: '100px',
                                              streamContent: true,
                                              sandboxed: true,
                                              transparent: true
                                            }

                               ]
                      }

                    ]
                },
                buttons: [{
                    type: "cancel",
                    name: "cancel",
                    text: "Cancel"
                }, {
                    type: "submit",
                    name: "save",
                    text: "Save",
                    primary: true
                }],
                 initialData: {
                     dimension: params.dimension,
                     content: params.content,
                     eccLevel: params.eccLevel,
                     outputType: params.outputType,
                     asDataURI: params.asDataURI,
                     outputBase64: params.outputBase64,
                     quietzoneSize: params.quietzoneSize,
                     qrcodepreview: '<div style="text-align: center; background-color: #E0E0E0;"><img src="' + 'ajax_qrcode.php?outputBase64=0&dimension=' + params.dimension + '&eccLevel=' + params.eccLevel + '&outputType=' + params.outputType + '&quietzoneSize=' + params.quietzoneSize + '&content=' + _encodeURI(params.content) + '&tinymce=6&' + tinymce.ckCsrfTokenName + '=' + tinymce.getCsrfToken() + '&nocache=' + (new Date().getTime() / 1000) + '"></div>'
                  },
                  onChange:function(api){
                    const data = api.getData();
                    data.dimension = parseInt(data.dimension);
                    if(data.dimension < 0)
                      data.dimension = 100;
                    if(data.dimension > 500)
                      data.dimension = 500;
                    api.setData( {dimension: data.dimension.toString()} );

                    data.quietzoneSize = parseInt(data.quietzoneSize);
                    if(data.quietzoneSize < 0){
                      data.quietzoneSize = 0;
                      api.setData( {quietzoneSize: data.quietzoneSize.toString()} );
                    }

                    if(data.placeholders != ""){
                      var elements = document.querySelectorAll(".tox-form");
                      var t;
                      if(elements.length == 1){
                        var form = elements[0];
                        if(form && form.childNodes.length > 0){
                          t = form.querySelector("textarea");
                        }
                      }

                      if(!t)
                       data.content += " " + data.placeholders;
                       else{
                         PasteTextToInputTextarea(t, data.placeholders); // paste on cursor pos
                         data.content = t.value; // take value for tinymce
                       }
                      api.setData( {placeholders: "", content: data.content} );
                    }

                    api.setData( {
                      qrcodepreview: '<div style="text-align: center; background-color: #E0E0E0;"><img src="' + './ajax_qrcode.php?outputBase64=0&dimension=' + data.dimension + '&eccLevel=' + data.eccLevel + '&outputType=' + data.outputType + '&quietzoneSize=' + data.quietzoneSize + '&content=' + _encodeURI(data.content) + '&tinymce=6&' + tinymce.ckCsrfTokenName + '=' + tinymce.getCsrfToken() + '&nocache=' + (new Date().getTime() / 1000) + '"></div>'
                    });

                  },
                  onCancel: function(api){
                  },
                  onSubmit: function(api) {
                    const data = api.getData();

                    editor.insertContent( '<img src="' + defaultQRCodeImage + '?asDataURI=' + data.asDataURI + '&outputBase64=0&dimension=' + data.dimension + '&eccLevel=' + data.eccLevel + '&outputType=' + data.outputType + '&quietzoneSize=' + data.quietzoneSize + '&content=' + _encodeURI(data.content) + '" width="' + data.dimension + '" height="' + data.dimension + '">' );

                    editor.setDirty(true);

                    api.close();
                }
            })

  }

  var qrcodeAllowed = function(element){
   if(!element) return false;

   if(qrcodeIsqrcode(element))
     return true;

   return !qrcode_notAllowedTags.includes(element.tagName);
  }

  var qrcodeIsqrcode = function(element){
   if(!element) return false;

   if(element.tagName == "IMG" && element.src.indexOf(defaultQRCodeImage + '?') >= 0)
     return true;

   return false;
  }

  editor.addCommand('mce_QRCode', (ui, v) => {

     element = editor.selection.getStart();

     if(qrcodeAllowed(element))
       qrcodeAction();

  });

  editor.ui.registry.addToggleButton('qrcodeButton', { //addToggleButton has setActive
    icon: "qrcode",
    tooltip: QRCodetooltip,
    onAction: () => {

        qrcodeAction();

    },
      onSetup: aMenuButton=>{
          const isEditable = ()=>{
              var sel = editor.selection.getStart();
              aMenuButton.setEnabled( editor.selection.isEditable() && qrcodeAllowed(sel)  );
              aMenuButton.setActive( editor.selection.isEditable() && qrcodeAllowed(sel) && qrcodeIsqrcode(sel)  );
          };
          return editor.on("NodeChange", isEditable),
          isEditable(),
          ()=>{
              editor.off("NodeChange", isEditable)
          }
      }

  });

  editor.ui.registry.addMenuItem('mnuQRCode', {
    text: QRCodeTitle,
    icon: "qrcode",
    onAction: () => {
      qrcodeAction();
    },
      onSetup: aMenuButton=>{
          const isEditable = ()=>{
              var sel = editor.selection.getStart();
              aMenuButton.setEnabled( editor.selection.isEditable() && qrcodeAllowed(sel)  );
          };
          return editor.on("NodeChange", isEditable),
          isEditable(),
          ()=>{
              editor.off("NodeChange", isEditable)
          }
     }

  });

  editor.ui.registry.addContextMenu('mnuQRCode', {
      update: (element) => ( qrcodeAllowed(element) ? 'mnuQRCode' : '' ) });

  return {
    name: 'QRCodePlugin',
    url: 'https://www.superwebmailer.de/'
  };
});

